from all_twitter_scraper.modules.Twitter_Scraper import Twitter_Scraper
from datetime import date
from all_twitter_scraper.modules.Criteria import Criteria


if __name__ == "__main__":
	criteria = Criteria()
	criteria.set_language('ar')
	criteria.set_since_date(date(2019, 1, 1))
	criteria.set_until_date(date(2020, 4, 1))
	criteria.set_all_of_keywords(['#covid_19'])
	twitter_scraper = Twitter_Scraper()
	twitter_scraper.scrap(criteria)



from lxml import etree
from datetime import datetime
from datetime import date


class Tweet:
	def __init__(self, tweet_dom: etree.HTML):
		self.__tweet_id = int
		self.__username = str
		self.__tweet_text = str
		self.__replies = int
		self.__retweets = int
		self.__likes = int
		self.__tweet_link = str
		self.__tweet_datetime = datetime
		self.__tweet_dom = tweet_dom
		self.__process_tweet_dom()
		# print('-----------------------------')
		# print(self.__tweet_id)
		# print(self.__username)
		# print(self.__tweet_text)
		# print(self.__replies)
		# print(self.__retweets)
		# print(self.__likes)
		# print(self.__tweet_link)
		# print(self.__tweet_datetime.strftime('%I:%M %p - %b %d, %Y'))
		# print('-----------------------------')

	def __process_tweet_dom(self):
			try:
				self.__tweet_id = self.__tweet_dom.xpath('.//div[contains(@class, "tweet")]')[0].get("data-tweet-id")
			except:
				self.__tweet_id = ''

			try:
				self.__username = self.__tweet_dom.xpath('.//div[@class="stream-item-header"]//span[contains(@class, "username")]')[0].xpath('string()').replace('@', '')
			except:
				self.__username = ''
			
			try:
				self.__tweet_text = self.__tweet_dom.xpath('.//p[contains(@class, "tweet-text")]')[0].xpath('string()')
			except:
				self.__tweet_text = ''

			try:
				self.__replies = int(self.__tweet_dom.xpath('.//span[contains(@class, "reply")]/span[@class="ProfileTweet-actionCount"]')[0].get("data-tweet-stat-count").replace(",", ""))
			except:
				self.__replies = 0

			try:
				self.__retweets = int(self.__tweet_dom.xpath('.//span[contains(@class, "retweet")]/span[@class="ProfileTweet-actionCount"]')[0].get("data-tweet-stat-count").replace(",", ""))
			except:
				self.__retweets = 0

			try:
				self.__likes = int(self.__tweet_dom.xpath('.//span[contains(@class, "favorite")]/span[@class="ProfileTweet-actionCount"]')[0].get("data-tweet-stat-count").replace(",", ""))
			except:
				self.__likes = 0

			try:
				self.__tweet_link = 'https://twitter.com' + self.__tweet_dom.xpath('.//div[contains(@class, "tweet")]')[0].get("data-permalink-path")
			except:
				self.__tweet_link = ''
				
			# tweet.author_id = int(tweetPQ("a.js-user-profile-link").attr("data-user-id"))
			try:
				datetime_string = self.__tweet_dom.xpath('.//small[@class="time"]/a')[0].get("title")
				self.__tweet_datetime = datetime.strptime(datetime_string, '%I:%M %p - %d %b %Y')
			except:
				self.__tweet_datetime = date(1, 1, 1)

			# tweet.hashtags, tweet.mentions = TweetManager.getHashtagsAndMentions(tweetPQ)

			# geoSpan = tweetPQ('span.Tweet-geo')
			# if len(geoSpan) > 0:
			# 	tweet.geo = geoSpan.attr('title')
			# else:
			# 	tweet.geo = ''

			# urls = []
			# for link in tweetPQ("a"):
			# 	try:
			# 		urls.append((link.attrib["data-expanded-url"]))
			# 	except KeyError:
			# 		pass

			# tweet.urls = ",".join(urls)

	def get_tweet_id(self):
		return self.__tweet_id

	def get_username(self):
		return self.__username

	def get_tweet_text(self):
		return self.__tweet_text

	def get_replies(self):
		return self.__replies

	def get_retweets(self):
		return self.__retweets

	def get_likes(self):
		return self.__likes

	def get_tweet_link(self):
		return self.__tweet_link

	def get_tweet_datetime(self):
		return self.__tweet_datetime




CHROME_WINDOWS_USER_AGENT = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.149 Safari/537.36'
FIREFOX_WINDOWS_USER_AGENT = 'Mozilla/5.0 (Windows NT 6.1; WOW64; rv:54.0) Gecko/20100101 Firefox/74.0'
INTERNET_EXPLORER_USER_AGENT = 'Mozilla/5.0 (Windows NT 10.0; Trident/7.0; rv:11.0) like Gecko'
CHROME_MAC_USER_AGENT = 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_6) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/80.0.3987.149 Safari/537.36'

USER_AGENTS = [
	CHROME_MAC_USER_AGENT,
	CHROME_WINDOWS_USER_AGENT,
	FIREFOX_WINDOWS_USER_AGENT,
	INTERNET_EXPLORER_USER_AGENT
]

HEADERS = {
	'Host': "twitter.com",
	'User-Agent': '',
	'Accept': "application/json, text/javascript, */*; q=0.01",
	'Accept-Language': "en-US,en;q=0.5",
	'X-Requested-With': "XMLHttpRequest",
	'Referer': '',
	'Connection': "keep-alive"
}



import requests
from lxml import etree
import json
from datetime import date
from .Criteria import Criteria
from .Tweet import Tweet
import time
import random
import csv
import os
from .constants import HEADERS
from .helpers import User_Agent_Rotator


class Twitter_Scraper:
	def __init__(self):
		self.__user_agent_rotator = User_Agent_Rotator()
		if not os.path.exists('log'):
			os.mkdir('log')
		self.__log_file = open(os.path.join('log', 'log.txt'), 'a', encoding='utf-8')

	def scrap(self, criteria: Criteria, last_maximum_position='', return_tweets_list=False):
		tweets = []
		counter = 0
		maximum_position = last_maximum_position
		output_file = open(str(criteria) + '.csv', 'w', encoding='utf-8', newline='')
		csv_writer = csv.DictWriter(output_file, ['Tweet body', 'Date', 'Likes', 'Retweets', 'Tweet link', 'Twitter link'])
		csv_writer.writeheader()
		output_file.flush()
		while True:
			criteria.set_maximum_position(maximum_position)
			search_url = criteria.generate_search_url()
			HEADERS['Referer'] = search_url
			json_response = self.get_json_response(search_url, HEADERS)
			self.__dump_json(json_response, os.path.join('log', 'last_response.json'))
			try:
				if json_response['new_latent_count'] <= 0:
					break
			except:
				break
			html_content = json_response["items_html"]
			dom = etree.HTML
			try:
				dom = etree.HTML(html_content)
			except:
				continue
			min_position = json_response["min_position"]
			self.__log('last min_position : ' + min_position + '\n')
			tweet_doms = dom.xpath('//li[contains(@class, "js-stream-item")]')
			new_tweets = []
			for tweet_dom in tweet_doms:
				tweet = Tweet(tweet_dom)
				new_tweets.append(tweet)
			if len(new_tweets) == 0:
				break
			for new_tweet in new_tweets:
				csv_writer.writerow({
					'Tweet body': new_tweet.get_tweet_text(),
					'Date': new_tweet.get_tweet_datetime().strftime('%I:%M %p - %d %b %Y'),
					'Likes': new_tweet.get_likes(),
					'Retweets': new_tweet.get_retweets(),
					'Tweet link': new_tweet.get_tweet_link(),
					'Twitter link': 'https://twitter.com/' + new_tweet.get_username()
				})
				output_file.flush()
			if return_tweets_list:
				tweets.extend(new_tweets)
			counter += len(new_tweets)
			print('                                                                                ', end='\r')
			print('Total scraped tweets :', counter, end='\r')
			time.sleep(2)
			maximum_position = min_position
		print('\n')
		return tweets
	
	def get_json_response(self, url: str, headers: dict):
		json_response = {}
		for trial in range(10):
			try:
				headers['User-Agent'] = self.__user_agent_rotator.get_user_agent()
				json_response = requests.get(url, headers=headers, timeout=15).json()
				break
			except Exception as e:
				print(e)
				if trial > 9:
					print('Can not fetch more tweets.')
					raise Exception
				else:
					time.sleep(5)
		return json_response

	def __log(self, string: str):
		self.__log_file.write(string)
		self.__log_file.flush()

	def __dump_json(self, json_dict: dict, filename: str):
		with open(filename, 'w', encoding='utf-8') as file:
			json.dump(json_dict, file, indent=4)

if __name__ == "__main__":
	criteria = Criteria()
	criteria.set_language('ar')
	criteria.set_since_date(date(2020, 3, 25))
	criteria.set_until_date(date(2020, 3, 26))
	criteria.set_search_keywords(['covid19'])
	twitter_scraper = Twitter_Scraper()
	twitter_scraper.scrap(criteria)



from .constants import *


class User_Agent_Rotator:
	def __init__(self):
		self.__index = 0

	def get_user_agent(self):
		user_agent = USER_AGENTS[self.__index]
		self.__index = (self.__index + 1) % len(USER_AGENTS)
		return user_agent



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

with open("requirements.txt", "r") as fh:
    INSTALL_REQUIRES = [l.split('#')[0].strip() for l in fh if not l.strip().startswith('#')]

setuptools.setup(
    name="all_twitter_scraper",
    version="0.0.4",
    author="Abdallah Samir",
    author_email="eng.abdallahsamir@gmail.com",
    license='MIT',
    description="Get all tweets filtered by the parameters that are in twitter advanced search.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/samirian/all_twitter_scraper",
    keywords="twitter tweets scraper",
    packages=setuptools.find_packages(),
    python_requires='>=3.4',
    install_requires=INSTALL_REQUIRES
)



from datetime import date
import requests


class Criteria:
	def __init__(self):
		self.__all_of_keywords = []
		self.__search_query = ''
		self.__since_date = date(1, 1, 1)
		self.__until_date = date(1, 1, 1)
		self.__language = ''
		self.__exact_phrase = ''
		self.__any_of_keywords = []
		self.__none_of_keywords = []
		self.__hashtags = []
		self.__maximum_position = ''
	
	def load_from_dict(self, criteria: dict):
		pass

	def set_since_date(self, since_date: date):
		self.__since_date = since_date

	def set_until_date(self, until_date: date):
		self.__until_date = until_date

	def set_search_query(self, search_query: str):
		self.__search_query = search_query

	def set_language(self, language: str):
		self.__language = language

	def set_maximum_position(self, maximum_position: str):
		self.__maximum_position = maximum_position

	def set_exact_phrase(self, phrase: str):
		self.__exact_phrase = phrase

	def set_any_of_keywords(self, keywords: list):
		self.__any_of_keywords = keywords

	def set_all_of_keywords(self, keywords: list):
		self.__all_of_keywords = keywords

	def set_none_of_keywords(self, keywords: list):
		self.__none_of_keywords = keywords

	def set_hashtags(self, hashtags: list):
		self.__hashtags = hashtags

	def generate_search_url(self):
		search_query = ''
		if len(self.__search_query) > 0:
			search_query = search_query
		else:
			if len(self.__all_of_keywords) > 0:
				search_query += ' '.join(self.__all_of_keywords)
			if len(self.__exact_phrase) > 0:
				search_query += ' "' + self.__exact_phrase + '"'
			if len(self.__any_of_keywords) > 0:
				search_query += ' (' + ' OR '.join(self.__any_of_keywords) + ')'
			if len(self.__none_of_keywords) > 0:
				search_query += ' -' + ' -'.join(self.__none_of_keywords)
			if len(self.__hashtags) > 0:
				search_query += ' (' + ' OR '.join(self.__hashtags) + ')'
			search_query = requests.utils.quote(search_query)
		filter_token = ''
		if self.__until_date.year > 1:
			filter_token += ' until:' + self.__until_date.strftime('%Y-%m-%d')
		if self.__since_date.year > 1:
			filter_token += ' since:' + self.__since_date.strftime('%Y-%m-%d')
		if len(self.__language) > 0:
			filter_token += ' lang:' + self.__language
		'""'
		filter_token = requests.utils.quote(filter_token)
		search_url = 'https://twitter.com/i/search/timeline?f=tweets&vertical=news&q=' + search_query + filter_token + '&src=typd&&include_available_features=1&include_entities=1&max_position=' + self.__maximum_position + '&reset_error_state=false'
		return search_url

	def __str__(self):
		return 'all_of_keywords [' + ' '.join(self.__all_of_keywords) + ']' + ' any_of_keywords [' + ' '.join(self.__any_of_keywords) + '] none_of_keywords [' + ' '.join(self.__none_of_keywords) + '] hashtags [' + ' '.join(self.__hashtags) + '] exact phrase ' + self.__exact_phrase + ' language ' + self.__language + ' since ' + self.__since_date.strftime('%Y-%m-%d') + ' until ' + self.__until_date.strftime('%Y-%m-%d')

if __name__ == "__main__":
	criteria = Criteria()
	# criteria.set_since_date(date(2019, 9, 9))
	# criteria.set_until_date(date(2020, 2, 2))
	criteria.set_all_of_keywords(['covid19', 'corona'])
	criteria.set_any_of_keywords(['covid19', 'corona'])
	criteria.set_none_of_keywords(['covid19', 'corona'])
	criteria.set_hashtags(['#covid19', '#corona'])
	criteria.set_exact_phrase('covid19 corona')
	print(criteria.generate_search_url())
	print(criteria)
